var buster = require("buster");
var assert = buster.assert;
buster.scriptLoader = require("../lib/buster-script-loader");
var fs = require("fs");

buster.testCase("Script loader create test", {
    "should create loader": function () {
        var loader = buster.scriptLoader.create();

        assert.isObject(loader);
        assert.isFunction(loader.onLoad);
    }
});

buster.testCase("Script loader add test", {
    setUp: function () {
        this.stub(fs, "readFile");
        this.loader = buster.scriptLoader.create();
    },

    "should call onLoad callback with context": function (done) {
        this.loader.onLoad(function (ctx) {
            assert.equals(ctx.meaningOfLife, 42);
            done();
        });

        this.loader.add("some-file.js");
        this.loader.load();
        assert.equals(fs.readFile.args[0][0], "some-file.js");

        fs.readFile.args[0][2](null, "var meaningOfLife = 42;");
    },

    "should load scripts sequentially": function (done) {
        this.loader.onLoad(function (ctx) {
            assert.equals(ctx.meaningOfLife, ctx.otherStuff);
            done();
        });

        this.loader.add("some-file.js");
        this.loader.add("other-file.js");
        this.loader.load();

        fs.readFile.args[1][2](null, "var otherStuff = meaningOfLife;");
        fs.readFile.args[0][2](null, "var meaningOfLife = 42;");
    },

    "should fire all callbacks when scripts load": function (done) {
        var callback = this.spy();
        this.loader.onLoad(callback);

        this.loader.onLoad(function (ctx) {
            assert(callback.called);
            done();
        });

        this.loader.add("some-file.js");
        this.loader.add("other-file.js");
        this.loader.load();

        fs.readFile.args[0][2](null, "var meaningOfLife = 42;");

        process.nextTick(function () {
            fs.readFile.args[1][2](null, "var otherStuff = meaningOfLife;");
        });
    },

    "should fire onLoad when calling load after scripts are read": function (done) {
        this.loader.onLoad(function (ctx) {
            assert.equals(ctx.meaningOfLife, ctx.otherStuff);
            done();
        });

        this.loader.add("some-file.js");
        this.loader.add("other-file.js");

        fs.readFile.args[0][2](null, "var meaningOfLife = 42;");
        var loader = this.loader;

        process.nextTick(function () {
            fs.readFile.args[1][2](null, "var otherStuff = meaningOfLife;");
            loader.load();
        });
    },

    "should trigger all callbacks on load": function (done) {
        var callback = this.spy();
        this.loader.onLoad(callback);

        this.loader.onLoad(function (ctx) {
            assert(callback.called);
            done();
        });

        this.loader.add("some-file.js");
        this.loader.add("other-file.js");

        fs.readFile.args[0][2](null, "var meaningOfLife = 42;");
        var loader = this.loader;

        process.nextTick(function () {
            fs.readFile.args[1][2](null, "var otherStuff = meaningOfLife;");
            loader.load();
        });
    },

    "should trigger all callbacks when files are loaded with a pause": function (done) {
        var callback = this.spy();
        this.loader.onLoad(callback);

        this.loader.add("some-file.js");
        fs.readFile.args[0][2](null, "var meaningOfLife = 42;");
        var loader = this.loader;

        process.nextTick(function () {
            loader.add("other-file.js");

            loader.onLoad(function (ctx) {
                assert(callback.called);
                done();
            });

            fs.readFile.args[1][2](null, "var otherStuff = meaningOfLife;");
            loader.load();
        });
    },

    "should use provided context": function (done) {
        var context = { id: 42 };
        var loader = buster.scriptLoader.create(context);

        loader.onLoad(function (ctx) {
            assert.equals(ctx.meaningOfLife, 42);
            done();
        });

        loader.add("some-file.js");
        fs.readFile.args[0][2](null, "var meaningOfLife = id;");
        loader.load();
    },

    "should expose common globals to script": function (done) {
        var context = {};
        var loader = buster.scriptLoader.create(context);

        loader.onLoad(function (ctx) {
            assert.equals(ctx.setTimeout, setTimeout);
            assert.equals(ctx.clearTimeout, clearTimeout);
            assert.equals(ctx.setInterval, setInterval);
            assert.equals(ctx.clearInterval, clearInterval);
            done();
        });

        loader.add("some-file.js");
        fs.readFile.args[0][2](null, "");
        loader.load();
    },

    "should add inline script": function (done) {
        var context = { id: 42 };
        var loader = buster.scriptLoader.create(context);

        loader.onLoad(function (ctx) {
            assert.equals(ctx.theTruth, 42);
            done();
        });

        loader.add("some-file.js");
        fs.readFile.args[0][2](null, "var meaningOfLife = id;");
        loader.addScript("var theTruth = meaningOfLife;");
        loader.load();
    }
});
